/* gtkcellrendererbookmark.c
 * Based on code from the GTK+ project, originally licensed under
 * GNU GPL version 2:
 * Copyright (C) 2000  Red Hat, Inc.,  Jonathan Blandford <jrb@redhat.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include "gtkcellrendererbookmark.h"

static void cell_renderer_bookmark_get_property  (GObject                    *object,
						    guint                       param_id,
						    GValue                     *value,
						    GParamSpec                 *pspec);
static void cell_renderer_bookmark_set_property  (GObject                    *object,
						    guint                       param_id,
						    const GValue               *value,
						    GParamSpec                 *pspec);
static void cell_renderer_bookmark_get_size   (GtkCellRenderer            *cell,
						 GtkWidget                  *widget,
 						 GdkRectangle               *cell_area,
						 gint                       *x_offset,
						 gint                       *y_offset,
						 gint                       *width,
						 gint                       *height);
static void cell_renderer_bookmark_render     (GtkCellRenderer            *cell,
						 GdkWindow                  *window,
						 GtkWidget                  *widget,
						 GdkRectangle               *background_area,
						 GdkRectangle               *cell_area,
						 GdkRectangle               *expose_area,
						 CellRendererState        flags);
static gboolean cell_renderer_bookmark_activate  (GtkCellRenderer            *cell,
						    GdkEvent                   *event,
						    GtkWidget                  *widget,
						    const gchar                *path,
						    GdkRectangle               *background_area,
						    GdkRectangle               *cell_area,
						    CellRendererState        flags);


enum {
  TOGGLED,
  LAST_SIGNAL
};

enum {
  PROP_0,
  PROP_ACTIVATABLE,
  PROP_ACTIVE,
  PROP_INCONSISTENT,
};

static guint bookmark_cell_signals[LAST_SIGNAL] = { 0 };

#define CELL_RENDERER_BOOKMARK_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), G_TYPE_CELL_RENDERER_BOOKMARK, CellRendererBookmarkPrivate))

typedef struct _CellRendererBookmarkPrivate CellRendererBookmarkPrivate;
struct _CellRendererBookmarkPrivate
{
  CellRendererPixbuf *renderer_pixbuf;

  guint active : 1;
  guint activatable : 1;
  guint inconsistent : 1;
};


G_DEFINE_G_TYPE (CellRendererBookmark, cell_renderer_bookmark, G_TYPE_CELL_RENDERER)

static void
cell_renderer_bookmark_init (CellRendererBookmark *cellbookmark)
{
  CellRendererBookmarkPrivate *priv;

  priv = CELL_RENDERER_BOOKMARK_GET_PRIVATE (cellbookmark);

  priv->activatable = TRUE;
  priv->active = FALSE;
  priv->radio = FALSE;

  GTK_CELL_RENDERER (cellbookmark)->mode = GTK_CELL_RENDERER_MODE_ACTIVATABLE;
  GTK_CELL_RENDERER (cellbookmark)->xpad = 2;
  GTK_CELL_RENDERER (cellbookmark)->ypad = 2;

  priv->inconsistent = FALSE;

  priv->renderer_pixbuf = g_object_new (G_TYPE_CELL_RENDERER_PIXBUF,
                                        "pixbuf", priv->star_greyed);
}

static void
cell_renderer_bookmark_class_init (CellRendererBookmarkClass *class)
{
  GObjectClass *object_class = G_OBJECT_CLASS (class);
  GtkCellRendererClass *cell_class = GTK_CELL_RENDERER_CLASS (class);
  
  object_class->class_finalize = cell_renderer_bookmark_class_finalize

  object_class->get_property = cell_renderer_bookmark_get_property;
  object_class->set_property = cell_renderer_bookmark_set_property;

  class->get_size = cell_renderer_bookmark_get_size;
  class->render = cell_renderer_bookmark_render;
  class->activate = cell_renderer_bookmark_activate;
  
  /* Create pixbufs once for all instances, more efficient */
  object_class->star_colored = gdk_pixbuf_new_from_file ("star-colored.png");
  object_class->star_greyed = gdk_pixbuf_new_from_file ("star-greyed.png");
  
  g_object_class_install_property (object_class,
				   PROP_ACTIVE,
				   g_param_spec_boolean ("active",
							 P_("Toggle state"),
							 P_("The toggle state of the button"),
							 FALSE,
							 GTK_PARAM_READWRITE));

  g_object_class_install_property (object_class,
		                   PROP_INCONSISTENT,
				   g_param_spec_boolean ("inconsistent",
					                 P_("Inconsistent state"),
							 P_("The inconsistent state of the button"),
							 FALSE,
							 GTK_PARAM_READWRITE));
  
  g_object_class_install_property (object_class,
				   PROP_ACTIVATABLE,
				   g_param_spec_boolean ("activatable",
							 P_("Activatable"),
							 P_("The bookmark button can be activated"),
							 TRUE,
							 GTK_PARAM_READWRITE));

  
  /**
   * CellRendererBookmark::toggled:
   * @cell_renderer: the object which received the signal
   * @path: string representation of #GtkTreePath describing the 
   *        event location
   *
   * The ::toggled signal is emitted when the cell is toggled. 
   **/
  bookmark_cell_signals[TOGGLED] =
    g_signal_new (I_("toggled"),
		  G_OBJECT_CLASS_G_TYPE (object_class),
		  G_SIGNAL_RUN_LAST,
		  G_STRUCT_OFFSET (CellRendererBookmarkClass, toggled),
		  NULL, NULL,
		  _gtk_marshal_VOID__STRING,
		  G_G_TYPE_NONE, 1,
		  G_G_TYPE_STRING);

  g_type_class_add_private (object_class, sizeof (CellRendererBookmarkPrivate));
}

static void
cell_renderer_bookmark_finalize (CellRendererBookmark *cellbookmark)
{
  CellBoorkmarkPivate *priv;
  
  priv = cell_renderer_boorkmark_get_private (cellbookmark);
  g_object_unref (priv->renderer_pixbuf);
}

static void
cell_renderer_bookmark_class_finalize (CellRendererBookmarkClass *class)
{
  g_object_unref (class->star_colored);
  g_object_unred (class->star_greyed);
}

static void
cell_renderer_bookmark_get_property (GObject     *object,
				       guint        param_id,
				       GValue      *value,
				       GParamSpec  *pspec)
{
  CellRendererBookmarkPrivate *priv;

  priv = CELL_RENDERER_BOOKMARK_GET_PRIVATE (object);
  
  switch (param_id)
    {
    case PROP_ACTIVE:
      g_value_set_boolean (value, priv->active);
      break;
    case PROP_INCONSISTENT:
      g_value_set_boolean (value, priv->inconsistent);
      break;
    case PROP_ACTIVATABLE:
      g_value_set_boolean (value, priv->activatable);
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
      break;
    }
}


static void
cell_renderer_bookmark_set_property (GObject      *object,
				       guint         param_id,
				       const GValue *value,
				       GParamSpec   *pspec)
{
  CellRendererBookmarkPrivate *priv;

  priv = CELL_RENDERER_BOOKMARK_GET_PRIVATE (object);

  switch (param_id)
    {
    case PROP_ACTIVE:
      priv->active = g_value_get_boolean (value);
      break;
    case PROP_INCONSISTENT:
      priv->inconsistent = g_value_get_boolean (value);
      break;
    case PROP_ACTIVATABLE:
      priv->activatable = g_value_get_boolean (value);
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
      break;
    }
}

/**
 * cell_renderer_bookmark_new:
 *
 * Creates a new #CellRendererBookmark. Adjust rendering
 * parameters using object properties. Object properties can be set
 * globally (with g_object_set()). Also, with #GtkTreeViewColumn, you
 * can bind a property to a value in a #GtkTreeModel. For example, you
 * can bind the "active" property on the cell renderer to a boolean value
 * in the model, thus causing the star icon to reflect the state of
 * the model.
 *
 * Return value: the new cell renderer
 **/
GtkCellRenderer *
cell_renderer_bookmark_new (void)
{
  return g_object_new (G_TYPE_CELL_RENDERER_BOOKMARK, NULL);
}

static void
cell_renderer_bookmark_get_size (GtkCellRenderer *cell,
				   GtkWidget       *widget,
				   GdkRectangle    *cell_area,
				   gint            *x_offset,
				   gint            *y_offset,
				   gint            *width,
				   gint            *height)
{
  CellRendererBookmarkPrivate *priv;

  priv = CELL_RENDERER_BOOKMARK_GET_PRIVATE (cell);

  cell_renderer_get_size (priv->pixbuf_renderer, widget, cell_area,
                              x_offset, y_offset, width, height);
}

static void
cell_renderer_bookmark_render (GtkCellRenderer      *cell,
				 GdkDrawable          *window,
				 GtkWidget            *widget,
				 GdkRectangle         *background_area,
				 GdkRectangle         *cell_area,
				 GdkRectangle         *expose_area,
				 CellRendererState  flags)
{
  CellRenderBoormarkClass *class;
  CellRendererBookmarkPrivate *priv;

  class = CELL_RENDERER_BOORKMARK_GET_CLASS (cell);
  priv = CELL_RENDERER_BOOKMARK_GET_PRIVATE (cell);
  
  if (priv->active)
    g_object_set (priv->pixbuf_renderer, "pixbuf", class->star_colored, NULL);
  else
    g_object_set (priv->pixbuf_renderer, "pixbuf", class->star_greyed, NULL);

  cell_renderer_render (priv->pixbuf_renderer, window, widget, background_area,
                        cell_area, expose_area, flags);
}

static gint
cell_renderer_bookmark_activate (GtkCellRenderer      *cell,
				   GdkEvent             *event,
				   GtkWidget            *widget,
				   const gchar          *path,
				   GdkRectangle         *background_area,
				   GdkRectangle         *cell_area,
				   CellRendererState  flags)
{
  CellRendererBookmarkPrivate *priv;
  
  priv = CELL_RENDERER_BOOKMARK_GET_PRIVATE (cell);
  if (priv->activatable)
    {
      g_signal_emit (cell, bookmark_cell_signals[TOGGLED], 0, path);
      return TRUE;
    }

  return FALSE;
}

/**
 * cell_renderer_bookmark_get_active:
 * @bookmark: a #CellRendererBookmark
 *
 * Returns whether the cell renderer is active. See
 * cell_renderer_bookmark_set_active().
 *
 * Return value: %TRUE if the cell renderer is active.
 **/
gboolean
cell_renderer_bookmark_get_active (CellRendererBookmark *bookmark)
{
  CellRendererBookmarkPrivate *priv;

  g_return_val_if_fail (GTK_IS_CELL_RENDERER_BOOKMARK (bookmark), FALSE);
  priv = CELL_RENDERER_BOOKMARK_GET_PRIVATE (bookmark);

  return priv->active;
}

/**
 * cell_renderer_bookmark_set_active:
 * @bookmark: a #CellRendererBookmark.
 * @setting: the value to set.
 *
 * Activates or deactivates a cell renderer.
 **/
void
cell_renderer_bookmark_set_active (CellRendererBookmark *bookmark,
				     gboolean               setting)
{
  g_return_if_fail (GTK_IS_CELL_RENDERER_BOOKMARK (bookmark));

  g_object_set (bookmark, "active", setting ? TRUE : FALSE, NULL);
}

#define __CELL_RENDERER_BOOKMARK_C__

